/**
 * @file
 * Provides stripe attachment logic.
 */

(function ($, window, Drupal, drupalSettings, Stripe) {

  'use strict';

  var stripe = null;

  Drupal.theme.stripeSucceeded = function (str) {
    return '<div class="stripe messages messages--status">' + Drupal.checkPlain(str) + '</div>';
  };

  // Argument passed from InvokeCommand.
  $.fn.stripeUpdatePaymentIntent = function(argument) {
    console.log('stripeUpdatePaymentIntent is called.');
    // Set textfield's
    // value to the passed arguments.
    var $element = $('#' + argument.trigger);
    var $form = $element.closest('form');
    var client_secret = $('[name*="[stripe][client_secret]"]', $form).val();
    if (argument.error) {
      $form.data('drupal-stripe-submit', true);
      $form.find('.js-form-submit:not(.js-stripe-submit)').first().trigger('click');
      return;
    }
    var paymentRequest = $element.data('drupal-stripe-pr');
    if (paymentRequest) {
      $element.removeData('drupal-stripe-pr');
      if (argument.total) {
        paymentRequest.update({total: argument.total});
      }
      paymentRequest.show();
    }

    var card = $element.data('drupal-stripe-card');
    if (card) {
      $element.removeData('drupal-stripe-card');
      stripe.confirmCardPayment(client_secret, {
        payment_method: {
          card: card,
          billing_details: argument.billing_details
        }
      })
      .then(function(result) {
        if (result.error) {
          // Show error to your customer
          $element.trigger('stripe:error', result.error.message);
        } else {
          // Send the token to your server
          console.log(result);
          $form.data('drupal-stripe-submit', true);
          $form.find('.js-form-submit:not(.js-stripe-submit)').first().trigger('click');
        }
      });
    }
  };

  /**
   * Attaches the behavior for the card element
   *
   * @type {Drupal~behavior}
   *
   * @prop {Drupal~behaviorAttach} attach
   */
  Drupal.behaviors.stripeCard = {
    attach: function (context, settings) {

      // Create a Stripe client
      if (drupalSettings.stripe && drupalSettings.stripe.apiKey && !stripe) {
        stripe = Stripe(drupalSettings.stripe.apiKey);
      }

      // Stripe was not initialized, do nothing.
      if (!stripe) {
        return;
      }
      for (var base in settings.stripe.elements) {
        var $element = $('#' + base, context);
        if (!$element.length) {
          continue;

        }
        // Isolate each iteration of the for loop above for this one stripe
        // element
        (function ($element, elementSettings) {
          var $form = $element.closest('form');

          // Adding a stripe processing class using our custom events
          $form.on('stripe:submit:start', function(e) {
            $(this).addClass('stripe-processing');
          });

          $form.on('stripe:submit:stop', function(e) {
            $(this).removeClass('stripe-processing');
          });


          var client_secret = $('[name*="[stripe][client_secret]"]', $form).val();
          var $payment_intent = $('#' + $element.attr('id') + '-stripe-payment-intent');

          if ($payment_intent.data('payment-intent-status') == 'succeeded') {
            $('#' + $element.attr('id') + '-stripe-card').after(Drupal.theme('stripeSucceeded', 'You have already been charged, please submit the rest of the form.'));
            return;
          }

          // Make sure we only process one stripe element per form
          if (!$form.data('drupal-stripe')) {
            $form.data('drupal-stripe', $element);

            // Create an instance of Elements
            var elements = stripe.elements();

            var cardOptions = {};

            // Allow other modules to change these options
            $element.trigger('stripe:element:create', {
              type: 'card',
              stripe: stripe,
              elements: elements,
              options: cardOptions
            });

            // Create an instance of the card Element
            var card = elements.create('card', cardOptions);

            // Allow other modules to act on the created card element
            $element.trigger('stripe:element:created', {
              type: 'card',
              stripe: stripe,
              elements: elements,
              options: cardOptions,
              card: card,
              settings: settings.stripe.elements[base]
            });

            // Add an instance of the card Element into the `card-element` <div>
            card.mount('#' + $element.attr('id') + '-stripe-card');

            $element.bind('stripe:error', function(event, text) {
              var displayError = document.getElementById($element.attr('id') + '-stripe-errors');
              $form.removeAttr('data-stripe-form-submit-last');
              $form.trigger('stripe:submit:stop');
              displayError.textContent = text;
            })

            // Handle real-time validation errors from the card Element.
            card.on('change', function(event) {
              $element.trigger('stripe:error', event.error ? event.error.message : "");
            });

            if (elementSettings.paymentrequest) {
              var prOptions = {};
              var paymentRequest = stripe.paymentRequest({
                country: elementSettings.country,
                currency: elementSettings.currency,
                total: {
                  label: elementSettings.label,
                  amount: elementSettings.amount,
                },
                requestPayerName: true,
                requestPayerEmail: true,
              });

              prOptions.paymentRequest = paymentRequest;

              // Allow other modules to change these options
              $element.trigger('stripe:element:create', {
                type: 'paymentRequestButton',
                stripe: stripe,
                elements: elements,
                options: prOptions
              });

              // Create an instance of the PaymentRequest Element
              var prButton = elements.create('paymentRequestButton', prOptions);

              // Allow other modules to act on the created PaymentRequest element
              $element.trigger('stripe:element:created', {
                type: 'paymentRequestButton',
                stripe: stripe,
                elements: elements,
                options: prOptions,
                prButton: prButton,
                settings: settings.stripe.elements[base]
              });

              // Check the availability of the Payment Request API first.
              paymentRequest.canMakePayment().then(function(result) {
                if (result) {
                  prButton.mount('#' + $element.attr('id') + '-stripe-paymentrequest');

                  prButton.on('click', function(event) {
                    event.preventDefault();
                    if (HTMLFormElement.prototype.reportValidity) {
                      if (!$form[0].reportValidity()) {
                        return false;
                      }
                    }
                    $element.data('drupal-stripe-pr', paymentRequest);
                    $form.trigger('stripe:submit:start');
                    $('#' + $element.attr('id') + '-stripe-update').trigger('mousedown');
                  });
                } else {
                  // document.getElementById('payment-request-button').style.display = 'none';
                }
              });

              paymentRequest.on('cancel', function() {
                $form.trigger('stripe:submit:stop');
              });

              paymentRequest.on('paymentmethod', function(ev) {
                // Confirm the PaymentIntent without handling potential next actions (yet).
                stripe.confirmCardPayment(
                  client_secret,
                  {payment_method: ev.paymentMethod.id},
                  {handleActions: false}
                ).then(function(confirmResult) {
                  if (confirmResult.error) {
                    // Report to the browser that the payment failed, prompting it to
                    // re-show the payment interface, or show an error message and close
                    // the payment interface.
                    $element.trigger('stripe:error', confirmResult.error.message);
                    ev.complete('fail');
                    $form.trigger('stripe:submit:stop');
                  } else {
                    // Report to the browser that the confirmation was successful, prompting
                    // it to close the browser payment method collection interface.
                    ev.complete('success');
                    // Check if the PaymentIntent requires any actions and if so let Stripe.js
                    // handle the flow. If using an API version older than "2019-02-11" instead
                    // instead check for: `paymentIntent.status === "requires_source_action"`.
                    if (confirmResult.paymentIntent.status === "requires_action") {
                      // Let Stripe.js handle the rest of the payment flow.
                      stripe.confirmCardPayment(client_secret).then(function(result) {
                        if (result.error) {
                          $element.trigger('stripe:error', result.error.message);
                          // The payment failed -- ask your customer for a new payment method.
                        } else {
                          // The payment has succeeded.
                          $form.data('drupal-stripe-submit', true);
                          $form.find('.js-form-submit:not(.js-stripe-submit)').first().trigger('click');
                        }
                      });
                    } else {
                      // The payment has succeeded.
                      $form.data('drupal-stripe-submit', true);
                      $form.find('.js-form-submit:not(.js-stripe-submit)').first().trigger('click');
                    }
                  }
                });
              });
            }

            $form.on('submit.drupalStripe', function(event) {
              var $form = $(event.currentTarget);

              if ($form.data('drupal-stripe-submit')) {
                // We were told to submit, at this point form should validate
                // and the card has been sucessfully charged.
                return true;
              }

              var formValues = $form.find(':input').not($element).serialize();
              var previousValues = $form.attr('data-stripe-form-submit-last');
              event.preventDefault();
              // event.currentTarget.submit();
              // @TODO: Check if this is actually necessary.
              // Using the same approach as drupal own double submit prevention
              // @see core/drupal.form
              if (previousValues !== formValues) {
                console.log('submitting');
                $form.attr('data-stripe-form-submit-last', formValues);

                $form.trigger('stripe:submit:start');
                $('#' + $element.attr('id') + '-stripe-update').trigger('mousedown');
                $element.data('drupal-stripe-card', card);

                // $form.trigger('submit', [true]);
              }
              else {
                console.log('prevent double submit');
              }
            });
          }

        })($element, settings.stripe.elements[base]);
      }
    }
  };

})(jQuery, window, Drupal, drupalSettings, Stripe);
