<?php

namespace Drupal\stripe\Element;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Render\Element\FormElement;
use Drupal\Core\Url;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Element\Hidden;
use Drupal\stripe\Event\StripeEvents;
use Drupal\stripe\Event\StripePaymentEvent;

/**
 * Provides a form element that will be rendered by stripe elements.
 *
 * @see https://stripe.com/docs/elements
 *
 * Usage example:
 * @code
 * @endcode
 * *
 * @FormElement("stripe")
 */
class Stripe extends FormElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    $info = [
      '#process' => [
        [$class, 'processAjaxForm'],
        [$class, 'processStripe'],
      ],
      '#theme_wrappers' => [
        'form_element' => [],
      ],
      '#stripe_amount' => 100,
      '#stripe_label' => 'Total',
      '#stripe_currency' => 'usd',
      '#stripe_country' => 'US',
      '#stripe_paymentrequest' => FALSE,
      '#stripe_paymentintent' => [],
    ];

    return $info;
  }

  /**
   * Processes a stripe element.
   */
  public static function processStripe(&$element, FormStateInterface $form_state, &$complete_form) {
    $config = \Drupal::config('stripe.settings');
    $apikey = $config->get('apikey.' . $config->get('environment') . '.public');
    $apikeySecret = $config->get('apikey.' . $config->get('environment') . '.secret');

    if (empty($apikey) || empty($apikeySecret)) {
      $settings_uri = Url::fromRoute('stripe.settings')->toString();
      $error = t('You must <a href="@settings_uri">configure</a> the <b>API Keys</b> on <b>%environment</b> for the stripe element to work.', ['%environment' => $config->get('environment'), '@settings_uri' => $settings_uri]);
      \Drupal::messenger()->addError($error);
      $element['error'] = [
        '#markup' => $error,
      ];
      return $element;
    }

    $stripe = new \Stripe\StripeClient($apikeySecret);

    $id = $element['#id'];
    $wrapper_id = 'stripe-' . implode('-', $element['#parents']) . '-wrapper';
    $button_name = 'stripe-' . implode('-', $element['#parents']) . '-button';

    $element['#tree'] = TRUE;


    $element['card'] = [
      '#markup' => "<div id=\"$id-stripe-card\"></div>",
    ];

    $element['paymentrequest'] = [
      '#markup' => "<div id=\"$id-stripe-paymentrequest\"></div>",
      '#access' => $element['#stripe_paymentrequest'],
    ];

    $element['errors'] = [
      '#markup' => "<div id=\"$id-stripe-errors\" class=\"stripe-errors\" role=\"alert\"></div>",
    ];

    $element['stripe'] = [
      '#type' => 'container',
      '#id' => $id,
    ];

    $element['stripe']['update'] = [
      '#id' => "$id-stripe-update",
      '#type' => 'submit',
      '#value' => t('Update'),
      // We specifically don't us a validate ajax callback as we want to act
      // only when the whole form validation succeddes, not otherwise
      // '#validate' => [[get_called_class(), 'validateStripeElementCallback']],
      '#submit' => [[get_called_class(), 'submitStripeElementCallback']],
      '#ajax' => [
        'callback' => [get_called_class(), 'ajaxStripeElementCallback'],
        'progress' => ['type' => 'none'],
      ],
      // Disable validation, hide button, add submit button trigger class.
      '#attributes' => [
        'formnovalidate' => 'formnovalidate',
        'class' => [
          'js-hide',
          'js-stripe-submit',
        ],
      ],
      // Issue #1342066 Document that buttons with the same #value need a unique
      // #name for the Form API to distinguish them, or change the Form API to
      // assign unique #names automatically.
      '#name' => $button_name,
    ];

    $element['stripe']['#attached']['library'][] = 'stripe/stripe';
    $element['stripe']['#attached']['drupalSettings']['stripe']['apiKey'] = $apikey;

    $settings = [];

    if (!isset($element['#value']['payment_intent'])) {
      $payment_intent_create = $element['#stripe_paymentintent'] ?? [];
      $payment_intent_create += [
        'amount' => is_numeric($element['#stripe_amount']) ? $element['#stripe_amount'] : 100,
        'currency' => $element['#stripe_currency'],
      ];
      $payment_intent = $stripe->paymentIntents->create($payment_intent_create);
    }
    else {
      $payment_intent = $stripe->paymentIntents->retrieve($element['#value']['payment_intent']);
    }

    $element['payment_intent'] = [
      '#type' => 'hidden',
      '#value' => $payment_intent->id,
      '#attributes' => [
        'id' => "$id-stripe-payment-intent",
        'data-payment-intent-status' => $payment_intent->status,
      ],
    ];
    $element['stripe']['client_secret'] = [
      '#type' => 'hidden',
      '#value' => $payment_intent->client_secret,
    ];
    $element['element_id'] = [
      '#type' => 'hidden',
      '#value' => $element['#id'],
    ];

    $settings['paymentrequest'] = $element['#stripe_paymentrequest'];
    $settings['country'] = $element['#stripe_country'];
    $settings['currency'] = $element['#stripe_currency'];
    $settings['amount'] = $element['#stripe_amount'];
    $settings['label'] = $element['#stripe_label'];

    $element['stripe']['#attached']['drupalSettings']['stripe']['elements'][$element['#id']] = $settings;

    // Add validate callback.
    $element['stripe'] += ['#element_validate' => []];
    array_unshift($element['stripe']['#element_validate'], [get_called_class(), 'validateStripeElement']);

    return $element;
  }

  /**
   * Validates an other element.
   */
  public static function validateStripeElement(&$element, FormStateInterface $form_state, &$form) {
    // Remove unnecessary submitted data
    // @TODO: Move it elsewhere or use something different?.
    $form_state->unsetValue($element['#parents']);
  }

  /**
   * Webform computed element submit callback.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public static function submitStripeElementCallback(array $form, FormStateInterface $form_state) {
    // Do nothing, but it's important to prevent other submission handlers
    // from being run
  }

  /**
   * Webform computed element Ajax callback.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The computed element element.
   */
  public static function ajaxStripeElementCallback(array $form, FormStateInterface $form_state) {
    $config = \Drupal::config('stripe.settings');
    $apikeySecret = $config->get('apikey.' . $config->get('environment') . '.secret');

    // Stripe form element from trigger
    $trigger = $form_state->getTriggeringElement();
    $key = array_slice($trigger['#parents'], -3, 1)[0];

    $payment_intent = $form_state->getValue([$key, 'payment_intent']);

    $response = new AjaxResponse();

    // Instantiate our event.
    $event = new StripePaymentEvent($form, $form_state, $key);

    // Dispatch our event to allow modules to update client-side elements of
    // the stripe elements
    $event_dispatcher = \Drupal::service('event_dispatcher');
    $event_dispatcher->dispatch(StripeEvents::PAYMENT, $event);

    $settings = [
      'trigger' => $form_state->getValue([$key, 'element_id']),
      'error' => $form_state->hasAnyErrors(),
    ];
    $total = $event->getTotal();
    if (!empty($total)) {
      $settings['total'] = $total;

      if ($payment_intent) {
        $stripe = new \Stripe\StripeClient($apikeySecret);
        $stripe->paymentIntents->update(
          $payment_intent,
          ['amount' => $total['amount']]
        );
      }
    }
    $billing_details = $event->getBillingDetails();
    if (!empty($billing_details)) {
      $settings['billing_details'] = $billing_details;
    }
    $response->addCommand(new InvokeCommand(NULL, 'stripeUpdatePaymentIntent', [$settings]));
    return $response;
  }

}
